@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qquickstateoperations_p.h"
 #include "qquickitem_p.h"
@@ -179,10 +143,6 @@ QQuickParentChange::QQuickParentChange(QObject *parent)
 {
 }
 
-QQuickParentChange::~QQuickParentChange()
-{
-}
-
 /*!
     \qmlproperty real QtQuick::ParentChange::x
     \qmlproperty real QtQuick::ParentChange::y
@@ -544,7 +504,7 @@ void QQuickParentChange::saveCurrentValues()
         return;
 
     QList<QQuickItem *> children = d->rewind->parent->childItems();
-    for (int ii = 0; ii < children.count() - 1; ++ii) {
+    for (int ii = 0; ii < children.size() - 1; ++ii) {
         if (children.at(ii) == d->target) {
             d->rewind->stackBefore = children.at(ii + 1);
             break;
@@ -850,10 +810,6 @@ QQuickAnchorChanges::QQuickAnchorChanges(QObject *parent)
 {
 }
 
-QQuickAnchorChanges::~QQuickAnchorChanges()
-{
-}
-
 QQuickAnchorChanges::ActionList QQuickAnchorChanges::actions()
 {
     Q_D(QQuickAnchorChanges);
@@ -1107,6 +1063,9 @@ void QQuickAnchorChanges::reverse()
                           stateHAnchors != QQuickAnchors::LeftAnchor &&
                           stateHAnchors != QQuickAnchors::RightAnchor &&
                           stateHAnchors != QQuickAnchors::HCenterAnchor);
+    // in case of an additive AnchorChange, we _did_ end up modifying the width
+    stateSetWidth |= ((stateHAnchors & QQuickAnchors::LeftAnchor) && (origHAnchors & QQuickAnchors::RightAnchor)) ||
+                     ((stateHAnchors & QQuickAnchors::RightAnchor) && (origHAnchors & QQuickAnchors::LeftAnchor));
     bool origSetWidth = (origHAnchors &&
                          origHAnchors != QQuickAnchors::LeftAnchor &&
                          origHAnchors != QQuickAnchors::RightAnchor &&
@@ -1122,12 +1081,15 @@ void QQuickAnchorChanges::reverse()
                            stateVAnchors != QQuickAnchors::BottomAnchor &&
                            stateVAnchors != QQuickAnchors::VCenterAnchor &&
                            stateVAnchors != QQuickAnchors::BaselineAnchor);
+    // in case of an additive AnchorChange, we _did_ end up modifying the height
+    stateSetHeight |= ((stateVAnchors & QQuickAnchors::TopAnchor) && (origVAnchors & QQuickAnchors::BottomAnchor)) ||
+                     ((stateVAnchors & QQuickAnchors::BottomAnchor) && (origVAnchors & QQuickAnchors::TopAnchor));
     bool origSetHeight = (origVAnchors &&
                           origVAnchors != QQuickAnchors::TopAnchor &&
                           origVAnchors != QQuickAnchors::BottomAnchor &&
                           origVAnchors != QQuickAnchors::VCenterAnchor &&
                           origVAnchors != QQuickAnchors::BaselineAnchor);
-    if (d->origHeight.isValid() && stateSetHeight && !origSetHeight && !!qt_is_nan(d->origHeight)) {
+    if (d->origHeight.isValid() && stateSetHeight && !origSetHeight && !qt_is_nan(d->origHeight)) {
         targetPrivate->heightValidFlag = true;
         if (targetPrivate->height != d->origHeight)
             targetPrivate->height.setValueBypassingBindings(d->origHeight);
@@ -1141,7 +1103,12 @@ void QQuickAnchorChanges::reverse()
 
     const QRectF newGeometry(d->target->position(), d->target->size());
     if (newGeometry != oldGeometry) {
-        targetPrivate->dirty(QQuickItemPrivate::Position);
+        QQuickItemPrivate::DirtyType dirtyFlags {};
+        if (newGeometry.topLeft() != oldGeometry.topLeft())
+            dirtyFlags = QQuickItemPrivate::DirtyType(dirtyFlags | QQuickItemPrivate::Position);
+        if (newGeometry.size() != oldGeometry.size())
+            dirtyFlags = QQuickItemPrivate::DirtyType(dirtyFlags | QQuickItemPrivate::Size);
+        targetPrivate->dirty(dirtyFlags);
         d->target->geometryChange(newGeometry, oldGeometry);
     }
 }
